/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
* Version: NPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Netscape Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/NPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is mozilla.org code.
*
* The Initial Developer of the Original Code is 
* Netscape Communications Corporation.
* Portions created by the Initial Developer are Copyright (C) 2002
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*   Geoff Beier <me@mollyandgeoff.com>
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or 
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the NPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the NPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

#import "TabButtonCell.h"
#import "TruncatingTextAndImageCell.h"
#import "BrowserTabBarView.h"

#define TAB_LEFTMARGIN 4 //distance between left edge and close button
#define TAB_CLOSE_BUTTON_PAD 2 //distance between close button and label
#define TAB_RIGHTMARGIN 4 //distance between favicon and right edge 
#define TAB_BOTTOMPAD 4 // number of pixels to offset from the bottom
#define TAB_SELECT_OFFSET 1 //number of pixels to drop everything down when selected

static NSImage * tabLeft = nil;
static NSImage * tabRight = nil;
static NSImage * activeTabBg = nil;

@interface TabButtonCell (PRIVATE)
- (void) loadImages;
@end

// these are the "tabs" for the tabless tab view
// to keep them lighter weight, they are not responsible for creating/maintaining much of anything
// the BrowserTabViewItem maintains the spinner, favicon, label and close button... this just positions
// them and brings them in and out of view, as well as painting itself and (TODO) handles dnd.
@implementation TabButtonCell
- (id)initFromTabViewItem:(BrowserTabViewItem *)tabViewItem
{
  [super init];
  mTabViewItem = [tabViewItem retain];
  //[self setMenu:[[mTabViewItem tabItemContentsView] menu]];
  return self;
}

// XXX does the menu need released here??
- (void)dealloc
{
  [super dealloc];
  [[mTabViewItem closeButton] removeFromSuperview];
  [mTabViewItem release];
}

- (BOOL)isSelected
{
  return (NSSelectedTab == [mTabViewItem tabState]);
}

- (BrowserTabViewItem *)tabViewItem
{
  return mTabViewItem;
}

// TODO
- (BOOL)willTrackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView
{
  return NO;
}

- (void)drawWithFrame:(NSRect)rect inView:(NSView *)controlView
{
  if( !tabLeft || !tabRight || !activeTabBg ) {
    [self loadImages];
  }
  // XXX is it worth it to see if the load failed? I don't think so, as if it did, the bundle is trashed and
  // we have bigger problems
  
  NSSize textSize = [mTabViewItem sizeOfLabel:NO];
  NSSize buttonSize = [[mTabViewItem closeButton] frame].size;
  
  // center based on the larger of the two heights if there's a difference
  float maxHeight = textSize.height > buttonSize.height ? textSize.height : buttonSize.height;
  NSRect buttonRect = NSMakeRect( rect.origin.x + TAB_LEFTMARGIN,
                                  rect.origin.y + (int)((rect.size.height - TAB_BOTTOMPAD - maxHeight)/2.0 + TAB_BOTTOMPAD),
                                  buttonSize.width, buttonSize.height);
  NSRect labelRect = NSMakeRect( NSMaxX(buttonRect) + TAB_CLOSE_BUTTON_PAD, 
                                rect.origin.y + (int)((rect.size.height - TAB_BOTTOMPAD - maxHeight)/2.0 + TAB_BOTTOMPAD),
                                rect.size.width - (NSWidth(buttonRect) +TAB_CLOSE_BUTTON_PAD+TAB_LEFTMARGIN+TAB_RIGHTMARGIN),
                                textSize.height );
  
  TruncatingTextAndImageCell *labelCell = [mTabViewItem labelCell];
  
  // make sure the close button and the favicon line up properly
  [labelCell setImagePadding:0.0];
  [labelCell setMaxImageHeight:buttonSize.height];
  
  
  if((NSSelectedTab == [mTabViewItem tabState])) {
    // move things down a little, to give the impression of being pulled forward
    labelRect.origin.y -= TAB_SELECT_OFFSET;
    buttonRect.origin.y -= TAB_SELECT_OFFSET;
    // XXX would it be better to maintain a CGContext and do a real gradient here?
    // it sounds heavier, but I haven't tested to be sure. This looks just as nice so long as
    // the tabbar height is fixed.
    [activeTabBg paintTiledInRect:rect];
    [tabLeft compositeToPoint:NSMakePoint( rect.origin.x, rect.origin.y )
                     operation:NSCompositeSourceOver];
    // I should probably come up with a better way to do this, but this is really cheap compared to 
    // any alternative I thought of... draw outside our cell's frame to make the border look better
    // next to the selected cell
    [tabRight compositeToPoint:NSMakePoint( NSMaxX(rect), rect.origin.y )
                                  operation:NSCompositeSourceOver];
  } else if([self mouseWithin] && ![self dragTarget]) {
    // XXX would it be better to paint an image here?
    [[NSColor colorWithCalibratedWhite:0.75 alpha:1.0] set];
    rect.origin.y += TAB_BOTTOMPAD;
    NSRectFill(rect);
  }
  if( [self dragTarget] ) {
    [[NSColor colorWithCalibratedRed:0.7 green:0.8 blue:1.0 alpha:1.0] set];
    rect.origin.y += TAB_BOTTOMPAD;
    NSRectFill(rect);
  }
  NSButton * closeButton  = [mTabViewItem closeButton];
  if( controlView != [closeButton superview] ) {
    [controlView addSubview:closeButton];
  }
  [closeButton setFrame:buttonRect];
  // XXX is this necessary, or even good?
  [closeButton setNeedsDisplay:YES];
  [labelCell drawInteriorWithFrame:labelRect inView:controlView];
}

- (void)addTrackingRectInView:(NSView *)aView withFrame:(NSRect)trackingRect cursorLocation:(NSPoint)currentLocation
{
  [super addTrackingRectInView:aView withFrame:trackingRect cursorLocation:currentLocation];
}

- (void)removeTrackingRectFromView:(NSView *)aView
{
  [super removeTrackingRectFromView:aView];
}


- (void)loadImages
{
  tabLeft = [[NSImage imageNamed:@"tab_left_side"] retain];
  tabRight = [[NSImage imageNamed:@"tab_right_side"] retain];
  activeTabBg = [[NSImage imageNamed:@"tab_active_bg"] retain];

}


@end
